% this script measures state durations from viterbi traces by counting 
% state occupancies within a sliding window of width 'tau'*'w' from 
% stimulus onset (plus an offset, 'offs') to a fixed time, 'tmax'.
%
% if the flag 'rxnmask' is activated, moving averages are taken from 
% stimulus onset to articulation offset, 'rxntimesoff'.
%
% created on March 30th, 2019, by Aram Giahi

% offs .............: offset relative to stimulus onset for start time
% tmax .............: offset relative to stimulus onset for end time (for
%                     state duration analysis
% trof1, trof2 .....: cutoff for the first and last trial
% w ................: half width of sliding window in units of time bins
% thr ..............: threshold state occupancies for state duration analysis
% rxnmask ..........: no fixed end time, but aligned to end of articulation
% rxntimesoff ......: end of articulation for each trial
% vi ...............: viterbi trace

offs=0; trof1=2; trof2=0; w=15; tmax=3200; thr=.25; rxnmask=0;

if rxnmask, w=5;end;

if ~rxnmask % state duration analysis
    r33=rxntimes(1+trof1:ntr-trof2); t=zeros(ntr-trof1-trof2,tmax/tau-offs);se=zeros(ntr,ns);
    sumvi=zeros(ntr-trof1-trof2,ns-1);
    for k=1+trof1:ntr-trof2
        for s=1:ns
            for l=ceil(offs/tau)+w+1:(1-rxnmask)*tmax/tau+rxnmask*(rxntimesoff(rxnfilter(k))+500-offset)/tau-w
                if (sum(vi(k,l-w:l+w)==seq(s))/(2*w))>(1-rxnmask)*thr+rxnmask*(sum(vi(k,l-w:l+w)~=seq(s))/(2*w))
                    t(k-trof1,l)=seq(s);
                    se(k-trof1,seq(s))=se(k-trof1,seq(s))+1;
                end
            end;
        end
    end
end

cr=zeros(1,ns);
for i=1:ns
    good{i}=find(sum(t==seq(i),2)>0);
    cr(i)=corr(squeeze(sum(t(good{i},:)==seq(i),2)),r33(good{i}));
end
cr

if ~rxnmask
    for i=2:ns
        figure;
        scatter(r33(good{seq(i)}),tau*se(good{seq(i)},seq(i)),100,'filled','MarkerEdgeColor',[0 114 189]/255,'MarkerFaceColor',[0 114 189]/255)
        xlim([0 2800]);ylim([0 2800]);axis square;
    end
end

mplot(t) % plot smoothed Viterbi trace
line((500-offset)/tau+rxntimesoff(rxnfilter(trof1+1:ntr))/tau,1:ntr-trof2-trof1,'color','white','LineWidth',2)
line((500-offset)/tau+r33/tau,1:ntr-trof2-trof1,'color','white','LineWidth',2)
%line(offs/tau+se(trof1+1:end,3)+se(trof1+1:end,2),trof1+1:ntr-trof2,'color','green','LineWidth',1)

lst=zeros(size(t,1),ns);
for i=1:size(t,1)
    for j=1:ns-1
        try
            lst(i,j)=find(t(i,:)==seq(j),1,'last');
        catch
            lst(i,j)=lst(i-1,j);
        end
    end
end

if rxnmask
    [cl,pl]=corr(lst(good{seq(3)},seq(3)),r33(good{seq(3)}))
    line(lst(:,seq(3)),trof1+1-2:ntr-trof2-2,'color','black','LineWidth',1)
    figure
    scatter(r33,tau*lst(:,seq(3)),100,'filled','MarkerEdgeColor',[0 114 189]/255,'MarkerFaceColor',[0 114 189]/255)
    %scatter(r33,tau*lst(:,seq(3)),100,'filled','MarkerEdgeColor',[217 83 25]/255,'MarkerFaceColor',[217 83 25]/255)
    xlim([0 2800]);ylim([0 2800]);axis square;
    if 1
        graphics
        line(lst(:,seq(3)),1+trof1:ntr-trof2,'color','black','LineWidth',1.5)
    end
end